import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { toAction } from 'vscode/vscode/vs/base/common/actions';
import { coalesce } from 'vscode/vscode/vs/base/common/arrays';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { fromNowByDay } from 'vscode/vscode/vs/base/common/date';
import { DisposableStore } from 'vscode/vscode/vs/base/common/lifecycle';
import { ThemeIcon } from 'vscode/vscode/vs/base/common/themables';
import { EditorAction2 } from 'vscode/vscode/vs/editor/browser/editorExtensions';
import { Position } from 'vscode/vscode/vs/editor/common/core/position';
import { SuggestController } from 'vscode/vscode/vs/editor/contrib/suggest/browser/suggestController';
import { localize2, localize } from 'vscode/vscode/vs/nls';
import { IActionViewItemService } from 'vscode/vscode/vs/platform/actions/browser/actionViewItemService.service';
import { DropdownWithPrimaryActionViewItem } from 'vscode/vscode/vs/platform/actions/browser/dropdownWithPrimaryActionViewItem';
import { Action2, MenuId, registerAction2, MenuRegistry, SubmenuItemAction, MenuItemAction } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IsWindowsContext, IsLinuxContext } from 'vscode/vscode/vs/platform/contextkey/common/contextkeys';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IOpenerService } from 'vscode/vscode/vs/platform/opener/common/opener.service';
import { IProductService } from 'vscode/vscode/vs/platform/product/common/productService.service';
import { IQuickInputService } from 'vscode/vscode/vs/platform/quickinput/common/quickInput.service';
import { ToggleTitleBarConfigAction } from 'vscode/vscode/vs/workbench/browser/parts/titlebar/titlebarActions';
import { IEditorGroupsService } from 'vscode/vscode/vs/workbench/services/editor/common/editorGroupsService.service';
import { ACTIVE_GROUP } from 'vscode/vscode/vs/workbench/services/editor/common/editorService';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService.service';
import { IExtensionsWorkbenchService } from 'vscode/vscode/vs/workbench/contrib/extensions/common/extensions.service';
import { ChatAgentLocation } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents';
import { IChatAgentService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents.service';
import { CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED, CONTEXT_CHAT_ENABLED, CONTEXT_CHAT_LOCATION, CONTEXT_IN_CHAT_INPUT, CONTEXT_CHAT_INPUT_CURSOR_AT_TOP, CONTEXT_IN_QUICK_CHAT, CONTEXT_IN_CHAT_SESSION, CONTEXT_CHAT_INSTALL_ENTITLED } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatContextKeys';
import { extractAgentAndCommand } from '../../common/chatParserTypes.js';
import { IChatService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatService.service';
import { isRequestVM } from '../../common/chatViewModel.js';
import { IChatWidgetHistoryService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatWidgetHistoryService.service';
import { showChatView, CHAT_VIEW_ID } from '../chat.js';
import { IChatWidgetService } from 'vscode/vscode/vs/workbench/contrib/chat/browser/chat.service';
import { ChatEditorInput } from '../chatEditorInput.js';
import { convertBufferToScreenshotVariable } from '../contrib/screenshot.js';
import { clearChatEditor } from './chatClear.js';
import product$1 from 'vscode/vscode/vs/platform/product/common/product';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { IHostService } from 'vscode/vscode/vs/workbench/services/host/browser/host.service';
import { isCancellationError } from 'vscode/vscode/vs/base/common/errors';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry.service';

const CHAT_CATEGORY = ( localize2(4614, 'Chat'));
const CHAT_OPEN_ACTION_ID = 'workbench.action.chat.open';
const defaultChat = {
    extensionId: product$1.defaultChatAgent?.extensionId ?? '',
    name: product$1.defaultChatAgent?.name ?? '',
    icon: Codicon[product$1.defaultChatAgent?.icon ?? 'commentDiscussion'],
    documentationUrl: product$1.defaultChatAgent?.documentationUrl ?? '',
    gettingStartedCommand: product$1.defaultChatAgent?.gettingStartedCommand ?? '',
};
class OpenChatGlobalAction extends Action2 {
    static { this.TITLE = ( localize2(4615, "Open Chat")); }
    constructor() {
        super({
            id: CHAT_OPEN_ACTION_ID,
            title: OpenChatGlobalAction.TITLE,
            icon: defaultChat.icon,
            f1: true,
            precondition: CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED,
            category: CHAT_CATEGORY,
            keybinding: {
                weight: 200 ,
                primary: 2048  | 512  | 39 ,
                mac: {
                    primary: 2048  | 256  | 39
                }
            },
            menu: {
                id: MenuId.ChatCommandCenter,
                group: 'a_chat',
                order: 1
            }
        });
    }
    async run(accessor, opts) {
        opts = typeof opts === 'string' ? { query: opts } : opts;
        const chatService = accessor.get(IChatService);
        const viewsService = accessor.get(IViewsService);
        const hostService = accessor.get(IHostService);
        const chatWidget = await showChatView(viewsService);
        if (!chatWidget) {
            return;
        }
        if (opts?.previousRequests?.length && chatWidget.viewModel) {
            for (const { request, response } of opts.previousRequests) {
                chatService.addCompleteRequest(chatWidget.viewModel.sessionId, request, undefined, 0, { message: response });
            }
        }
        if (opts?.attachScreenshot) {
            const screenshot = await hostService.getScreenshot();
            if (screenshot) {
                chatWidget.attachmentModel.addContext(convertBufferToScreenshotVariable(screenshot));
            }
        }
        if (opts?.query) {
            if (opts.isPartialQuery) {
                chatWidget.setInput(opts.query);
            }
            else {
                chatWidget.acceptInput(opts.query);
            }
        }
        chatWidget.focusInput();
    }
}
class ChatHistoryAction extends Action2 {
    constructor() {
        super({
            id: `workbench.action.chat.history`,
            title: ( localize2(4616, "Show Chats...")),
            menu: {
                id: MenuId.ViewTitle,
                when: ( (ContextKeyExpr.equals('view', CHAT_VIEW_ID))),
                group: 'navigation',
                order: 2
            },
            category: CHAT_CATEGORY,
            icon: Codicon.history,
            f1: true,
            precondition: CONTEXT_CHAT_ENABLED
        });
    }
    async run(accessor) {
        const chatService = accessor.get(IChatService);
        const quickInputService = accessor.get(IQuickInputService);
        const viewsService = accessor.get(IViewsService);
        const editorService = accessor.get(IEditorService);
        const showPicker = () => {
            const openInEditorButton = {
                iconClass: ThemeIcon.asClassName(Codicon.file),
                tooltip: ( localize(4617, "Open in Editor")),
            };
            const deleteButton = {
                iconClass: ThemeIcon.asClassName(Codicon.x),
                tooltip: ( localize(4618, "Delete")),
            };
            const renameButton = {
                iconClass: ThemeIcon.asClassName(Codicon.pencil),
                tooltip: ( localize(4619, "Rename")),
            };
            const getPicks = () => {
                const items = chatService.getHistory();
                items.sort((a, b) => (b.lastMessageDate ?? 0) - (a.lastMessageDate ?? 0));
                let lastDate = undefined;
                const picks = items.flatMap((i) => {
                    const timeAgoStr = fromNowByDay(i.lastMessageDate, true, true);
                    const separator = timeAgoStr !== lastDate ? {
                        type: 'separator', label: timeAgoStr,
                    } : undefined;
                    lastDate = timeAgoStr;
                    return [
                        separator,
                        {
                            label: i.title,
                            description: i.isActive ? `(${( localize(4620, 'current'))})` : '',
                            chat: i,
                            buttons: i.isActive ? [renameButton] : [
                                renameButton,
                                openInEditorButton,
                                deleteButton,
                            ]
                        }
                    ];
                });
                return coalesce(picks);
            };
            const store = ( (new DisposableStore()));
            const picker = store.add(quickInputService.createQuickPick({ useSeparators: true }));
            picker.placeholder = ( localize(4621, "Switch to chat"));
            const picks = getPicks();
            picker.items = picks;
            store.add(picker.onDidTriggerItemButton(async (context) => {
                if (context.button === openInEditorButton) {
                    const options = { target: { sessionId: context.item.chat.sessionId }, pinned: true };
                    editorService.openEditor({ resource: ChatEditorInput.getNewEditorUri(), options }, ACTIVE_GROUP);
                    picker.hide();
                }
                else if (context.button === deleteButton) {
                    chatService.removeHistoryEntry(context.item.chat.sessionId);
                    picker.items = getPicks();
                }
                else if (context.button === renameButton) {
                    const title = await quickInputService.input({ title: ( localize(4622, "New chat title")), value: context.item.chat.title });
                    if (title) {
                        chatService.setChatSessionTitle(context.item.chat.sessionId, title);
                    }
                    showPicker();
                }
            }));
            store.add(picker.onDidAccept(async () => {
                try {
                    const item = picker.selectedItems[0];
                    const sessionId = item.chat.sessionId;
                    const view = await viewsService.openView(CHAT_VIEW_ID);
                    view.loadSession(sessionId);
                }
                finally {
                    picker.hide();
                }
            }));
            store.add(picker.onDidHide(() => store.dispose()));
            picker.show();
        };
        showPicker();
    }
}
class OpenChatEditorAction extends Action2 {
    constructor() {
        super({
            id: `workbench.action.openChat`,
            title: ( localize2(4623, "Open Editor")),
            f1: true,
            category: CHAT_CATEGORY,
            precondition: CONTEXT_CHAT_ENABLED
        });
    }
    async run(accessor) {
        const editorService = accessor.get(IEditorService);
        await editorService.openEditor({ resource: ChatEditorInput.getNewEditorUri(), options: { pinned: true } });
    }
}
class ChatAddAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.chat.addParticipant',
            title: ( localize2(4624, "Chat with Extension")),
            icon: Codicon.mention,
            f1: false,
            category: CHAT_CATEGORY,
            menu: {
                id: MenuId.ChatInput,
                when: ( (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.Panel))),
                group: 'navigation',
                order: 1
            }
        });
    }
    async run(accessor, ...args) {
        const widgetService = accessor.get(IChatWidgetService);
        const context = args[0];
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        if (!widget) {
            return;
        }
        const hasAgentOrCommand = extractAgentAndCommand(widget.parsedInput);
        if (hasAgentOrCommand?.agentPart || hasAgentOrCommand?.commandPart) {
            return;
        }
        const suggestCtrl = SuggestController.get(widget.inputEditor);
        if (suggestCtrl) {
            const curText = widget.inputEditor.getValue();
            const newValue = curText ? `@ ${curText}` : '@';
            if (!curText.startsWith('@')) {
                widget.inputEditor.setValue(newValue);
            }
            widget.inputEditor.setPosition(( (new Position(1, 2))));
            suggestCtrl.triggerSuggest(undefined, true);
        }
    }
}
function registerChatActions() {
    registerAction2(OpenChatGlobalAction);
    registerAction2(ChatHistoryAction);
    registerAction2(OpenChatEditorAction);
    registerAction2(ChatAddAction);
    registerAction2(class ClearChatInputHistoryAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.clearInputHistory',
                title: ( localize2(4625, "Clear Input History")),
                precondition: CONTEXT_CHAT_ENABLED,
                category: CHAT_CATEGORY,
                f1: true,
            });
        }
        async run(accessor, ...args) {
            const historyService = accessor.get(IChatWidgetHistoryService);
            historyService.clearHistory();
        }
    });
    registerAction2(class ClearChatHistoryAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.clearHistory',
                title: ( localize2(4626, "Clear All Workspace Chats")),
                precondition: CONTEXT_CHAT_ENABLED,
                category: CHAT_CATEGORY,
                f1: true,
            });
        }
        async run(accessor, ...args) {
            const editorGroupsService = accessor.get(IEditorGroupsService);
            const viewsService = accessor.get(IViewsService);
            const chatService = accessor.get(IChatService);
            chatService.clearAllHistoryEntries();
            const chatView = viewsService.getViewWithId(CHAT_VIEW_ID);
            if (chatView) {
                chatView.widget.clear();
            }
            editorGroupsService.groups.forEach(group => {
                group.editors.forEach(editor => {
                    if (editor instanceof ChatEditorInput) {
                        clearChatEditor(accessor, editor);
                    }
                });
            });
        }
    });
    registerAction2(class FocusChatAction extends EditorAction2 {
        constructor() {
            super({
                id: 'chat.action.focus',
                title: ( localize2(4627, 'Focus Chat List')),
                precondition: ( (ContextKeyExpr.and(CONTEXT_IN_CHAT_INPUT))),
                category: CHAT_CATEGORY,
                keybinding: [
                    {
                        when: ( (ContextKeyExpr.and(
                            CONTEXT_CHAT_INPUT_CURSOR_AT_TOP,
                             (CONTEXT_IN_QUICK_CHAT.negate())
                        ))),
                        primary: 2048  | 16 ,
                        weight: 100 ,
                    },
                    {
                        when: ( (ContextKeyExpr.and(
                             (ContextKeyExpr.or(IsWindowsContext, IsLinuxContext)),
                             (CONTEXT_IN_QUICK_CHAT.negate())
                        ))),
                        primary: 2048  | 16 ,
                        weight: 100 ,
                    },
                    {
                        when: ( (ContextKeyExpr.and(CONTEXT_IN_CHAT_SESSION, CONTEXT_IN_QUICK_CHAT))),
                        primary: 2048  | 18 ,
                        weight: 200 ,
                    }
                ]
            });
        }
        runEditorCommand(accessor, editor) {
            const editorUri = editor.getModel()?.uri;
            if (editorUri) {
                const widgetService = accessor.get(IChatWidgetService);
                widgetService.getWidgetByInputUri(editorUri)?.focusLastMessage();
            }
        }
    });
    registerAction2(class FocusChatInputAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.focusInput',
                title: ( localize2(4628, "Focus Chat Input")),
                f1: false,
                keybinding: [
                    {
                        primary: 2048  | 18 ,
                        weight: 200 ,
                        when: ( (ContextKeyExpr.and(
                            CONTEXT_IN_CHAT_SESSION,
                             (CONTEXT_IN_CHAT_INPUT.negate()),
                             (CONTEXT_IN_QUICK_CHAT.negate())
                        ))),
                    },
                    {
                        when: ( (ContextKeyExpr.and(
                            CONTEXT_IN_CHAT_SESSION,
                             (CONTEXT_IN_CHAT_INPUT.negate()),
                            CONTEXT_IN_QUICK_CHAT
                        ))),
                        primary: 2048  | 16 ,
                        weight: 200 ,
                    }
                ]
            });
        }
        run(accessor, ...args) {
            const widgetService = accessor.get(IChatWidgetService);
            widgetService.lastFocusedWidget?.focusInput();
        }
    });
    registerAction2(InstallChatWithPromptAction);
    registerAction2(InstallChatWithoutPromptAction);
    registerAction2(LearnMoreChatAction);
}
function stringifyItem(item, includeName = true) {
    if (isRequestVM(item)) {
        return (includeName ? `${item.username}: ` : '') + item.messageText;
    }
    else {
        return (includeName ? `${item.username}: ` : '') + ( (item.response.toString()));
    }
}
MenuRegistry.appendMenuItem(MenuId.CommandCenter, {
    submenu: MenuId.ChatCommandCenter,
    title: ( localize(4629, "Chat")),
    icon: defaultChat.icon,
    when: ( (ContextKeyExpr.and(
         (ContextKeyExpr.has('config.chat.commandCenter.enabled')),
         (ContextKeyExpr.or(CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED, CONTEXT_CHAT_INSTALL_ENTITLED))
    ))),
    order: 10001,
});
registerAction2(class ToggleChatControl extends ToggleTitleBarConfigAction {
    constructor() {
        super('chat.commandCenter.enabled', ( localize(4630, 'Chat Controls')), ( localize(4631, "Toggle visibility of the Chat Controls in title bar")), 3, false, ( (ContextKeyExpr.and(
             (ContextKeyExpr.has('config.window.commandCenter')),
             (ContextKeyExpr.or(CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED, CONTEXT_CHAT_INSTALL_ENTITLED))
        ))));
    }
});
let ChatCommandCenterRendering = class ChatCommandCenterRendering {
    static { this.ID = 'chat.commandCenterRendering'; }
    constructor(actionViewItemService, agentService, instantiationService) {
        this._store = ( (new DisposableStore()));
        this._store.add(actionViewItemService.register(MenuId.CommandCenter, MenuId.ChatCommandCenter, (action, options) => {
            if (!(action instanceof SubmenuItemAction)) {
                return undefined;
            }
            const dropdownAction = toAction({
                id: 'chat.commandCenter.more',
                label: ( localize(4632, "More...")),
                run() { }
            });
            const chatExtensionInstalled = ( (agentService.getAgents().some(agent => agent.isDefault)));
            const primaryAction = instantiationService.createInstance(MenuItemAction, {
                id: chatExtensionInstalled ? CHAT_OPEN_ACTION_ID : InstallChatWithPromptAction.ID,
                title: chatExtensionInstalled ? OpenChatGlobalAction.TITLE : InstallChatWithPromptAction.TITLE,
                icon: defaultChat.icon,
            }, undefined, undefined, undefined, undefined);
            return instantiationService.createInstance(DropdownWithPrimaryActionViewItem, primaryAction, dropdownAction, action.actions, '', {
                ...options,
                skipTelemetry: true,
            });
        }, agentService.onDidChangeAgents));
    }
    dispose() {
        this._store.dispose();
    }
};
ChatCommandCenterRendering = ( (__decorate([
    ( (__param(0, IActionViewItemService))),
    ( (__param(1, IChatAgentService))),
    ( (__param(2, IInstantiationService)))
], ChatCommandCenterRendering)));
class BaseInstallChatAction extends Action2 {
    async run(accessor) {
        const extensionsWorkbenchService = accessor.get(IExtensionsWorkbenchService);
        const productService = accessor.get(IProductService);
        const telemetryService = accessor.get(ITelemetryService);
        const justification = this.getJustification(productService);
        let installResult;
        try {
            await extensionsWorkbenchService.install(defaultChat.extensionId, {
                justification,
                enable: true,
                installPreReleaseVersion: productService.quality !== 'stable'
            }, 15 );
            installResult = 'installed';
        }
        catch (error) {
            installResult = isCancellationError(error) ? 'cancelled' : 'failed';
        }
        telemetryService.publicLog2('commandCenter.chatInstall', {
            installResult,
            hasJustification: !!justification
        });
    }
}
class InstallChatWithPromptAction extends BaseInstallChatAction {
    static { this.ID = 'workbench.action.chat.installWithPrompt'; }
    static { this.TITLE = ( localize2(4633, "Install {0}", defaultChat.name)); }
    constructor() {
        super({
            id: InstallChatWithPromptAction.ID,
            title: InstallChatWithPromptAction.TITLE,
            icon: defaultChat.icon,
            category: CHAT_CATEGORY
        });
    }
    getJustification(productService) {
        return ( localize(
            4634,
            "AI features in {0} require this extension.",
            productService.nameShort
        ));
    }
}
class InstallChatWithoutPromptAction extends BaseInstallChatAction {
    static { this.ID = 'workbench.action.chat.installWithoutPrompt'; }
    static { this.TITLE = ( localize2(4633, "Install {0}", defaultChat.name)); }
    constructor() {
        super({
            id: InstallChatWithoutPromptAction.ID,
            title: InstallChatWithoutPromptAction.TITLE,
            category: CHAT_CATEGORY,
            menu: {
                id: MenuId.ChatCommandCenter,
                group: 'a_atfirst',
                order: 1,
                when: ( (CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED.negate()))
            }
        });
    }
    getJustification() {
        return undefined;
    }
}
class LearnMoreChatAction extends Action2 {
    static { this.ID = 'workbench.action.chat.learnMore'; }
    static { this.TITLE = ( localize2(4635, "Learn More")); }
    constructor() {
        super({
            id: LearnMoreChatAction.ID,
            title: LearnMoreChatAction.TITLE,
            category: CHAT_CATEGORY,
            menu: [{
                    id: MenuId.ChatCommandCenter,
                    group: 'a_atfirst',
                    order: 2,
                    when: ( (CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED.negate()))
                }, {
                    id: MenuId.ChatCommandCenter,
                    group: 'z_atlast',
                    order: 1,
                    when: CONTEXT_CHAT_PANEL_PARTICIPANT_REGISTERED
                }]
        });
    }
    async run(accessor) {
        const openerService = accessor.get(IOpenerService);
        if (defaultChat.documentationUrl) {
            openerService.open(( (URI.parse(defaultChat.documentationUrl))));
        }
    }
}

export { CHAT_CATEGORY, CHAT_OPEN_ACTION_ID, ChatCommandCenterRendering, registerChatActions, stringifyItem };
